# This is modified based on gdown (https://github.com/wkentaro/gdown)
# gdown is a Python package to download public or shared Google Drive files
# It does not need Google autentication, so access can be simplified
# We do not need the functionality of download. Just need to obtain file/folder
# names along with the file IDs
import requests
import urllib
import itertools
import json
import os.path as osp
import re
import warnings
import bs4
import polars as pl


MAX_NUMBER_FILES = 50

class FileURLRetrievalError(Exception):
    pass

class _GoogleDriveFile(object):
    TYPE_FOLDER = "application/vnd.google-apps.folder"

    def __init__(self, id, name, type, children=None):
        self.id = id
        self.name = name
        self.type = type
        self.children = children if children is not None else []

    def is_folder(self):
        return self.type == self.TYPE_FOLDER

def _get_session():
    sess = requests.session()
    # We need to use different user agent for folder download c.f., file
    user_agent = (
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) '
        'AppleWebKit/537.36 (KHTML, like Gecko) '
        'Chrome/98.0.4758.102 Safari/537.36')
    sess.headers.update({"User-Agent": user_agent})
    return sess

def _parse_google_drive_file(url, content):
    """Extracts information about the current page file and its children."""

    folder_soup = bs4.BeautifulSoup(content, features="html.parser")
    # finds the script tag with window['_DRIVE_ivd']
    encoded_data = None
    for script in folder_soup.select("script"):
        inner_html = script.decode_contents()
        if "_DRIVE_ivd" in inner_html:
            # first js string is _DRIVE_ivd, the second one is the encoded arr
            regex_iter = re.compile(r"'((?:[^'\\]|\\.)*)'").finditer(inner_html)
            # get the second elem in the iter
            try:
                encoded_data = (
                    next(itertools.islice(regex_iter, 1, None)).group(1))
            except StopIteration:
                raise RuntimeError("Couldn't find the folder encoded JS string")
            break

    if encoded_data is None:
        raise RuntimeError(
            "Cannot retrieve the folder information from the link. "
            "You may need to change the permission to "
            "'Anyone with the link', or have had many accesses. "
            "Check FAQ in https://github.com/wkentaro/gdown?tab=readme-ov-file#faq."
        )

    # decodes the array and evaluates it as a python array
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", category=DeprecationWarning)
        decoded = encoded_data.encode("utf-8").decode("unicode_escape")
        folder_arr = json.loads(decoded)

    folder_contents = [] if folder_arr[0] is None else folder_arr[0]
    sep = " - "  # unicode dash
    splitted = folder_soup.title.contents[0].split(sep)
    if len(splitted) >= 2:
        name = sep.join(splitted[:-1])
    else:
        raise RuntimeError(
            "file/folder name cannot be extracted from: {}".format(
                folder_soup.title.contents[0]))

    gdrive_file = _GoogleDriveFile(
        id=url.split("/")[-1], name=name, type=_GoogleDriveFile.TYPE_FOLDER)
    id_name_type_iter = [
        (e[0], e[2].encode("raw_unicode_escape").decode("utf-8"), e[3])
        for e in folder_contents]
    return gdrive_file, id_name_type_iter

def _download_and_parse_google_drive_link(sess, url):
    """Get folder structure of Google Drive folder URL."""

    for _ in range(2):
        # canonicalize the language into English
        if "?" in url:
            url += "&hl=en"
        else:
            url += "?hl=en"

        res = sess.get(url, verify=True)
        # need to try with canonicalized url if the original url redirects to gdrive
        url = res.url

    gdrive_file, id_name_type_iter = _parse_google_drive_file(
        url=url, content=res.text)
    for child_id, child_name, child_type in id_name_type_iter:
        # Skip folders named 'individual'
        if child_type == _GoogleDriveFile.TYPE_FOLDER and child_name in {'Individual', 'Results'}:
            continue

        if child_type != _GoogleDriveFile.TYPE_FOLDER:
            gdrive_file.children.append(
                _GoogleDriveFile(id=child_id, name=child_name, type=child_type))
        else:
            child = _download_and_parse_google_drive_link(
                sess=sess,
                url="https://drive.google.com/drive/folders/" + child_id)
            gdrive_file.children.append(child)

    return gdrive_file

def _get_individual_signal_folder_id(sess, url):
    """
    Recursively searches for a folder named 'Predictors' and returns its ID.
    If no such folder exists, returns None.
    """
    for _ in range(2):
        # Set URL to English if not already done
        url += "&hl=en" if "?" in url else "?hl=en"
        res = sess.get(url, verify=True)
        url = res.url  # update in case of redirect

    # Parse the Google Drive file and iterate over children
    gdrive_file, id_name_type_iter = _parse_google_drive_file(url=url, content=res.text)
    for child_id, child_name, child_type in id_name_type_iter:
        # Only return ID if folder is named 'Predictors'
        if child_type == _GoogleDriveFile.TYPE_FOLDER and child_name == "Predictors":
            return child_id  # Returns only the folder ID of 'Predictors'

        # If it's another folder, recursively search in it
        if child_type == _GoogleDriveFile.TYPE_FOLDER:
            result = _get_individual_signal_folder_id(
                sess=sess,
                url="https://drive.google.com/drive/folders/" + child_id
            )
            if result:
                return result

    return None

def _get_directory_structure(gdrive_file):
    """Converts a Google Drive folder structure into a local directory list."""

    directory_structure = []
    for file in gdrive_file.children:
        file.name = file.name.replace(osp.sep, "_")
        if file.is_folder():
            directory_structure.append((None, file.name))
            for i in _get_directory_structure(file):
                directory_structure.append(i)
        elif not file.children:
            directory_structure.append((file.id, file.name))

    return directory_structure

def _get_url_from_gdrive_confirmation(contents):
    url = ""
    for line in contents.splitlines():
        m = re.search(r'href="(\/uc\?export=download[^"]+)', line)
        if m:
            url = "https://docs.google.com" + m.groups()[0]
            url = url.replace("&amp;", "&")
            break
        soup = bs4.BeautifulSoup(line, features="html.parser")
        form = soup.select_one("#download-form")
        if form is not None:
            url = form["action"].replace("&amp;", "&")
            url_components = urllib.parse.urlsplit(url)
            query_params = urllib.parse.parse_qs(url_components.query)
            for param in form.findChildren("input", attrs={"type": "hidden"}):
                query_params[param["name"]] = param["value"]
                query = urllib.parse.urlencode(query_params, doseq=True)
                url = urllib.parse.urlunsplit(url_components._replace(query=query))
            break
        m = re.search('"downloadUrl":"([^"]+)', line)
        if m:
            url = m.groups()[0]
            url = url.replace("\\u003d", "=")
            url = url.replace("\\u0026", "&")
            break
        m = re.search('<p class="uc-error-subcaption">(.*)</p>', line)
        if m:
            error = m.groups()[0]
            raise FileURLRetrievalError(error)
    if not url:
        raise FileURLRetrievalError(
            "Cannot retrieve the public link of the file. "
            "You may need to change the permission to "
            "'Anyone with the link', or have had many accesses. "
            "Check FAQ in https://github.com/wkentaro/gdown?tab=readme-ov-file#faq."
        )
    return url

def _get_name_id_map(url):
    sess = _get_session()
    gdrive_file = _download_and_parse_google_drive_link(sess, url=url)
    directory_structure = _get_directory_structure(gdrive_file)
    url_prefix = 'https://drive.google.com/uc?id='
    datasets_map = {
        'SignalDoc.csv': 'signal_doc',
        'PredictorPortsFull.csv': 'op',
        'PredictorAltPorts_Deciles.zip': 'deciles_ew',
        'PredictorAltPorts_DecilesVW.zip': 'deciles_vw',
        'PredictorAltPorts_LiqScreen_ME_gt_NYSE20pct.zip': 'ex_nyse_p20_me',
        'PredictorAltPorts_LiqScreen_NYSEonly.zip': 'nyse',
        'PredictorAltPorts_LiqScreen_Price_gt_5.zip': 'ex_price5',
        'PredictorAltPorts_Quintiles.zip': 'quintiles_ew',
        'PredictorAltPorts_QuintilesVW.zip': 'quintiles_vw',
        'signed_predictors_dl_wide.zip': 'firm_char'
    }

    df = (
        pl.DataFrame(
            directory_structure, orient='row', schema=['file_id', 'name'])
        .filter(~pl.col('name').str.contains('xlsx$|docx$|txt'))
        .with_row_index()
        .with_columns(
            pl.when(
                (pl.col('file_id').is_null()) &
                (pl.col('file_id').shift(1).is_null()) &
                (pl.col('index')!=0)
            )
            .then(pl.col('name').shift(1)+'/'+pl.col('name'))
            .when(pl.col('file_id').is_null())
            .then(pl.col('name'))
            .alias('full_name')
        )
        .with_columns(
            pl.when(
                (pl.col('name')=='SignalDoc.csv') |
                (pl.col('full_name')==pl.col('name'))
            )
            .then(pl.col('name'))
            .otherwise(pl.col('full_name').forward_fill()+'/'+pl.col('name'))
            .alias('full_name')
        )
        .filter(
            (pl.col('file_id').is_not_null()) &
            (pl.col('full_name').str.contains('zip$|csv$'))
        )
        .select('name', 'full_name', 'file_id')
        .with_columns(
            file_id=url_prefix+pl.col('file_id'),
            download_name=
            pl.col('name').replace_strict(datasets_map, default=None)
        )
        .filter(pl.col('download_name').is_not_null())
    )

    # Individual signals
    signal_folder_id = _get_individual_signal_folder_id(sess, url)
    signal_folder_url = f'https://drive.google.com/embeddedfolderview?id={signal_folder_id}'
    signal_response = requests.get(signal_folder_url)
    signal_text = str(signal_response.content)

    signal_file_name = r'<div class="flip-entry-title">(.*?).csv</div>'
    signal_file_id = r'https://drive\.google\.com/file/d/([-\w]{25,})/view\?usp=drive_web'
    signal_matches = {
        'signal': re.findall(signal_file_name, signal_text),
        'file_id': re.findall(signal_file_id, signal_text)}
    df_signal = (
        pl.DataFrame(signal_matches)
        .with_columns(file_id='https://drive.google.com/uc?id='+pl.col('file_id'))
    )
    return df, df_signal

def _get_readable_link(url):
    sess = _get_session()
    res = sess.get(url, verify=True)
    readable_link = _get_url_from_gdrive_confirmation(res.text)
    return readable_link



from . import urls
from .gdrive_parse import _get_name_id_map, _get_readable_link
import polars as pl
import pandas as pd
import requests
from io import BytesIO
from zipfile import ZipFile
from tabulate import tabulate
import wrds
import re
import time


def list_release():
    df = pl.DataFrame(
        {'release':
         [re.findall(r'release(\d+)_url', i)[0]
          for i in dir(urls) if i.startswith('release')]}
    )
    table = [i for i in df.iter_rows()]
    headers = ['Release']
    print(tabulate(table, headers, tablefmt='simple_outline'))

class OpenAP:
    def __init__(self, release_year=None):
        if not release_year:
            release_url = getattr(urls, dir(urls)[-1], None)
        if release_year:
            release_url = getattr(urls, f'release{release_year}_url', None)

        self.name_id_map, self.individual_signal_id_map = _get_name_id_map(release_url)
        self.signal_sign = (
            pl.read_csv(
                self._get_url('signal_doc'), infer_schema_length=300,
                columns=['Acronym', 'Sign'], null_values='NA')
            .rename({'Acronym': 'signal', 'Sign': 'sign'})
            .with_columns(pl.col('sign').cast(pl.Int8))
        )

    def list_port(self):
        df = (
            self.name_id_map.select('name', 'download_name')
            .filter(pl.col('name').str.starts_with('Predictor'))
        )
        table = [i for i in df.iter_rows()]
        headers = ['CZ portfolio file', 'Name for download']
        print(tabulate(table, headers, tablefmt='simple_outline'))

    def _get_url(self, data_name):
        data_header = self.name_id_map.filter(pl.col('download_name')==data_name)
        file_with_confirm = ['firm_char', 'deciles_ew', 'deciles_vw']
        if data_name in file_with_confirm:
            self.url = _get_readable_link(data_header[0, 'file_id'])
        else:
            self.url = data_header[0, 'file_id']

        return self.url

    def _get_individual_signal_url(self, signal_name):
        data_header = self.individual_signal_id_map.filter(pl.col('signal')==signal_name)
        self.url = data_header[0, 'file_id']
        return self.url

    def _zip_source(self, url):
        # Reading in chunks is 20% faster for large single file
        chunk_size = 1024 * 1024 * 10
        source = requests.get(url, stream=True)
        io_data = BytesIO()
        for chunk in source.iter_content(chunk_size=chunk_size):
            if chunk:
                io_data.write(chunk)

        io_data.seek(0)
        zip_file = ZipFile(io_data)
        return zip_file

    def _convert_to_backend(self, df, df_backend):
        if df_backend == 'polars':
            return df
        if df_backend == 'pandas':
            return df.to_pandas()

    def _port_indiv(self, df, predictor):
        n_input = len(predictor)
        df = df.filter(pl.col('signalname').is_in(predictor))
        n = df['signalname'].n_unique()
        if n != n_input:
            print('One or more input predictors are not available.')

        return df

    def _dl_port_op(self, df_backend, predictor=None):
        if not predictor:
            df = (
                pl.read_csv(
                    self.url, null_values='NA',
                    schema_overrides={'port': pl.String})
                .with_columns(pl.col('date').str.to_date('%Y-%m-%d'))
            )

        if predictor:
            if type(predictor) is list:
                df = (
                    pl.read_csv(
                        self.url, null_values='NA',
                        schema_overrides={'port': pl.String})
                    .with_columns(pl.col('date').str.to_date('%Y-%m-%d'))
                )
                df = self._port_indiv(df, predictor)
            else:
                print('Predictor must be a list')

        df = df.sort('signalname', 'port', 'date')
        return self._convert_to_backend(df, df_backend)

    def _dl_port_alt(self, df_backend, predictor=None):
        if not predictor:
            zip_file = self._zip_source(self.url)
            df = (
                pl.read_csv(
                    zip_file.read(zip_file.filelist[0]), null_values='NA',
                    schema_overrides={'port': pl.String})
                .with_columns(pl.col('date').str.to_date('%Y-%m-%d'))
            )

        if predictor:
            if type(predictor) is list:
                zip_file = self._zip_source(self.url)
                df = (
                    pl.read_csv(
                        zip_file.read(zip_file.filelist[0]), null_values='NA',
                        schema_overrides={'port': pl.String})
                    .with_columns(pl.col('date').str.to_date('%Y-%m-%d'))
                )
                df = self._port_indiv(df, predictor)
            else:
                print('Predictor must be a list')

        df = df.sort('signalname', 'port', 'date')
        return self._convert_to_backend(df, df_backend)

    def _dl_signal_crsp3(self):
        conn = wrds.Connection()

        df = conn.raw_sql(
            """
            select permno, date, prc, ret, shrout
            from crsp.msf
            """, date_cols=['date']
        )

        # They are signed
        df = (
            pl.from_pandas(df)
            .select(
                pl.col('permno').cast(pl.Int32),
                pl.col('date').dt.year().mul(100)
                .add(pl.col('date').dt.month())
                .cast(pl.Int32).alias('yyyymm'),
                pl.col('prc').abs().log().mul(-1).alias('Price'),
                pl.col('prc').abs().mul(pl.col('shrout')).truediv(1000)
                .log().mul(-1).alias('Size'),
                pl.col('ret').fill_null(0).mul(-1).alias('STreversal')
            )
            .with_columns(
                pl.when(pl.col('Size').is_finite())
                .then(pl.col('Size'))
                .alias('Size')
            )
        )
        return df

    def _dl_signal(self, df_backend, predictor=None):
        if not predictor:
            temp = self._dl_signal_crsp3()
            zip_file = self._zip_source(self.url)
            df = (
                pl.read_csv(
                    zip_file.read(zip_file.filelist[0]), infer_schema_length=0)
                .with_columns(
                    pl.col('permno', 'yyyymm').cast(pl.Int32),
                    pl.exclude('permno', 'yyyymm').cast(pl.Float64))
                .join(temp, how='left', on=['permno', 'yyyymm'])
            )

        if predictor:
            crsp3 = {'Price', 'Size', 'STreversal'}
            ex_crsp3 = [i for i in predictor if i not in crsp3]
            if type(predictor) is list:
                try:
                    if crsp3 & set(predictor):
                        temp = self._dl_signal_crsp3()

                    if len(ex_crsp3) > 0:
                        zip_file = self._zip_source(self.url)
                        df = (
                            pl.from_pandas(
                                pd.read_csv(
                                    zip_file.open(zip_file.filelist[0]),
                                    usecols=['permno', 'yyyymm']+ex_crsp3,
                                    engine='pyarrow')
                            )
                            .with_columns(
                                pl.col('permno', 'yyyymm').cast(pl.Int32))
                        )
                        if len(ex_crsp3) < len(predictor):
                            df = df.join(
                                temp, how='left', on=['permno', 'yyyymm'])
                    else:
                        df = temp

                    df = (
                        df.select('permno', 'yyyymm', pl.col(predictor))
                        .filter(
                            pl.any_horizontal(pl.col(predictor)).is_not_null())
                        .with_columns(
                            pl.exclude('permno', 'yyyymm').cast(pl.Float64))
                        .sort('permno', 'yyyymm')
                    )
                except:
                    print('One or more input predictors are not available.')
            else:
                print('Predictor must be a list')

        df = df.sort('permno', 'yyyymm')
        return self._convert_to_backend(df, df_backend)

    def _dl_individual_signal(self, df_backend, predictor, signed=False):
        crsp3 = {'Price', 'Size', 'STreversal'}
        ex_crsp3 = [i for i in predictor if i not in crsp3]
        if type(predictor) is list:
            try:
                if crsp3 & set(predictor):
                    temp = self._dl_signal_crsp3()
                    # Price, Size and STreversal from CRSP by default are signed
                    # If signed = True, do nothing
                    # If signed = False, multiply -1
                    if not signed:
                        temp = temp.with_columns(
                            pl.col('Price').mul(-1),
                            pl.col('Size').mul(-1),
                            pl.col('STreversal').mul(-1))

                if len(ex_crsp3) > 0:
                    df = pl.DataFrame(
                        schema={'permno': pl.Int32, 'yyyymm': pl.Int32})
                    for i in ex_crsp3:
                        self.url = self._get_individual_signal_url(i)
                        temp_signal = pl.read_csv(self.url)
                        if len(temp_signal) > 0:
                            temp_signal = temp_signal.with_columns(
                                pl.col('permno', 'yyyymm').cast(pl.Int32))
                        if len(temp_signal) == 0:
                            self.url = _get_readable_link(self.url)
                            temp_signal = (
                                pl.read_csv(self.url)
                                .with_columns(
                                    pl.col('permno', 'yyyymm').cast(pl.Int32))
                            )
                        if signed:
                            _sign = (
                                self.signal_sign.filter(pl.col('signal')==i)
                                .get_column('sign')[0])
                            if _sign is not None:
                                temp_signal = (
                                    temp_signal.with_columns(pl.col(i)*_sign))

                        df = df.join(
                            temp_signal, how='full', on=['permno', 'yyyymm'],
                            coalesce=True)
                    if len(ex_crsp3) < len(predictor):
                        df = df.join(
                            temp, how='full', on=['permno', 'yyyymm'],
                            coalesce=True)
                else:
                    df = temp

                df = (
                    df.select('permno', 'yyyymm', pl.col(predictor))
                    .filter(pl.any_horizontal(pl.col(predictor)).is_not_null())
                    .with_columns(
                        pl.exclude('permno', 'yyyymm').cast(pl.Float64))
                    .sort('permno', 'yyyymm')
                )
            except:
                print('One or more input predictors are not available.')
        else:
            print('Predictor must be a list')

        df = df.sort('permno', 'yyyymm')
        return self._convert_to_backend(df, df_backend)

    def _print_time(self, time_used):
        if time_used <= 60:
            print(f'\nData is downloaded: {time_used:.0f}s')
        else:
            print(f'\nData is downloaded: {time_used/60:.0f} mins')

    def dl_signal_doc(self, df_backend):
        url = self._get_url('signal_doc')
        df = pl.read_csv(url, infer_schema_length=300)
        return self._convert_to_backend(df, df_backend)

    def dl_port(self, data_name, df_backend, predictor=None):
        port_alt_list = [
            'deciles_ew', 'deciles_vw', 'ex_nyse_p20_me', 'nyse', 'ex_price5',
            'quintiles_ew', 'quintiles_vw']

        if df_backend in ['polars', 'pandas']:
            self.url = self._get_url(data_name)
            if self.url:
                start_time = time.time()
                if data_name == 'op':
                    df = self._dl_port_op(df_backend, predictor)
                if data_name in port_alt_list:
                    df = self._dl_port_alt(df_backend, predictor)

                end_time = time.time()
                time_used = end_time - start_time
                self._print_time(time_used)
                return df
            else:
                raise ValueError('Dataset is not available.')
        else:
            raise ValueError("Unsupported backend. Choose 'polars' or 'pandas'.")

    def dl_all_signals(self, df_backend, predictor=None):
        if df_backend in ['polars', 'pandas']:
            self.url = self._get_url('firm_char')
            if self.url:
                start_time = time.time()
                df = self._dl_signal(df_backend, predictor)
                end_time = time.time()
                time_used = end_time - start_time
                self._print_time(time_used)
                return df
            else:
                raise ValueError('Dataset is not available.')
        else:
            raise ValueError("Unsupported backend. Choose 'polars' or 'pandas'.")

    def dl_signal(self, df_backend, predictor, signed=False):
        if df_backend in ['polars', 'pandas']:
            start_time = time.time()
            df = self._dl_individual_signal(df_backend, predictor, signed)
            end_time = time.time()
            time_used = end_time - start_time
            self._print_time(time_used)
            return df
        else:
            raise ValueError("Unsupported backend. Choose 'polars' or 'pandas'.")



import openassetpricing as oap


# Initialize OpenAP
openap = oap.OpenAP(2022)

# ==========
# List available datasets
# ==========
openap.list_datasets()



# ==========
# Download SignalDoc.csv in pandas dataframe
# ==========
df = openap.dl('signal_doc', 'pandas')



# ==========
# Portfolios -> Full Sets OP -> PredictorPortsFull.csv
# ==========

# Download entier file in polars dataframe
df = openap.dl('port_op', 'polars')

# Download specific predictors (can be single or multiple predictors)
df = openap.dl('port_op', 'polars', predictor=['AM'])
df = openap.dl('port_op', 'polars', predictor=['AM', 'Mom12m'])



# ==========
# Portfolios -> Full Sets Alt -> PredictorAltPorts_Deciles.zip
# Portfolios -> Full Sets Alt -> PredictorAltPorts_DecilesVW.zip
# Portfolios -> Full Sets Alt -> PredictorAltPorts_LiqScreen_NYSEonly.zip
# ==========

# Download entier file
df = openap.dl('port_deciles_ew', 'polars')
df = openap.dl('port_deciles_vw', 'polars')
df = openap.dl('port_nyse', 'polars')

# Download specific predictors (can be single or multiple predictors)
df = openap.dl('port_deciles_ew', 'polars', predictor=['Accruals'])
df = openap.dl('port_deciles_ew', 'polars', predictor=['BM', 'Mom6m'])



# ==========
# Firm Level Characteristics -> Full Sets -> signed_predictors_dl_wide.zip
# ==========

# Download all firm characteristics
df = openap.dl('char_predictors', 'polars')

# Download specific firm characteristics
df = openap.dl('char_predictors', 'polars', ['BM'])



# Data is from Chen and Zimmermann: https://www.openassetpricing.com/

# Version 2.00: data release - 2025.10
release202510_url = (
    'https://drive.google.com/drive/folders/1qQDuTsnyvWfEJR6nPBQZ8xxlq6bkLG_y')

# Version 1.41: data release - 2024.10
release202410_url = (
    'https://drive.google.com/drive/folders/1SSoHGbwgyhRwUCzLE0YWvUlS0DjLCd4k')

# Version 1.40: data release - 2024.08
release202408_url = (
    'https://drive.google.com/drive/folders/1-PqsR-tOjv3-U9DRHw85X-VznYlu-Sfc')

# Version 1.30: data release - 2023.08
release2023_url = (
    'https://drive.google.com/drive/folders/1EP6oEabyZRamveGNyzYU0u6qJ-N43Qfq')

# Version 1.20: data release - 2022.03
release2022_url = (
    'https://drive.google.com/drive/folders/1O18scg9iBTiBaDiQFhoGxdn4FdsbMqGo')



"""
Inputs: None; this script relies on setuptools metadata defined inline.
Outputs: Distributes the openassetpricing package with optional extras for examples.
How to run: Execute `pip install .` for the core install. To include example dependencies, run
"pip install '.[examples]'" (quote the extras when using shells like zsh).
"""

from setuptools import setup, find_packages

setup(
    name='openassetpricing',
    version='0.0.2',
    author='Peng Li, Andrew Chen, Tom Zimmermann',
    author_email='pl750@bath.ac.uk, andrew.y.chen@frb.gov, tom.zimmermann@uni-koeln.de',
    license='GPLv2',
    packages=find_packages(),
    install_requires=[
        'polars',
        'pandas',
        'requests',
        'tabulate',
        'wrds',
        'pyarrow',
        'beautifulsoup4'
    ],
    extras_require={
        'examples': [
            'matplotlib',
            'numpy',
            'scikit-learn',
            'seaborn',
            'statsmodels'
        ]
    },
)



import openassetpricing as oap


# List available release versions
oap.list_release()

# Initialize OpenAP with most recent release
openap = oap.OpenAP()

# ==========
# List available portfolios
# ==========
openap.list_port()



# ==========
# Download SignalDoc.csv in pandas dataframe
# ==========
df = openap.dl_signal_doc('pandas')



# ==========
# Portfolios -> Full Sets OP -> PredictorPortsFull.csv
# ==========

# Download entier file in polars dataframe
df = openap.dl_port('op', 'polars')

# Download specific predictors (can be single or multiple predictors)
df = openap.dl_port('op', 'polars', ['AM'])
df = openap.dl_port('op', 'polars', ['AM', 'Mom12m'])



# ==========
# Portfolios -> Full Sets Alt -> PredictorAltPorts_Deciles.zip
# Portfolios -> Full Sets Alt -> PredictorAltPorts_DecilesVW.zip
# Portfolios -> Full Sets Alt -> PredictorAltPorts_LiqScreen_NYSEonly.zip
# ==========

# Download entier file
df = openap.dl_port('deciles_ew', 'polars')
df = openap.dl_port('deciles_vw', 'polars')
df = openap.dl_port('nyse', 'pandas')

# Download specific predictors (can be single or multiple predictors)
df = openap.dl_port('deciles_ew', 'polars', ['Accruals'])
df = openap.dl_port('deciles_ew', 'polars', ['BM', 'Mom6m'])



# ==========
# Firm Level Characteristics
# ==========

# Download all firm characteristics (signed)
df = openap.dl_all_signals('polars')

# Download specific firm characteristics
df = openap.dl_signal('polars', ['BM'])
